import { type DOMElement } from './dom.js';
export declare const renderNodeToScreenReaderOutput: (node: DOMElement, options?: {
    parentRole?: string;
    skipStaticElements?: boolean;
}) => string;
