import React, { type ReactNode } from 'react';
import { type Styles } from '../styles.js';
export type Props = {
    readonly children: ReactNode;
    readonly width: number;
    readonly style?: Styles;
};
export default function StaticRender({ children, width, style }: Props): React.JSX.Element;
