import { type StyledLine } from './styled-line.js';
import { type Region } from './output.js';
import { type DOMElement, type StickyHeader } from './dom.js';
import { type Selection } from './selection.js';
type Result = {
    output: string;
    outputHeight: number;
    staticOutput: string;
    styledOutput: StyledLine[];
    cursorPosition?: {
        row: number;
        col: number;
    };
    stickyHeaders: StickyHeader[];
    root?: Region;
};
declare const renderer: (node: DOMElement, options: {
    isScreenReaderEnabled: boolean;
    selection?: Selection;
    selectionStyle?: (line: StyledLine, index: number) => void;
    skipScrollbars?: boolean;
    trackSelection?: boolean;
}) => Result;
export default renderer;
