/**
 * @license
 * Copyright 2026 Google LLC
 * SPDX-License-Identifier: Apache-2.0
 */
import { StyledLine } from './styled-line.js';
export declare const sliceStyledChars: (line: StyledLine, begin: number, end?: number) => StyledLine;
export declare const truncateStyledChars: (line: StyledLine, columns: number, options?: {
    position?: "start" | "middle" | "end";
}) => StyledLine;
export declare const wrapStyledChars: (line: StyledLine, columns: number) => StyledLine[];
export declare const wrapOrTruncateStyledChars: (line: StyledLine, maxWidth: number, textWrap?: string) => StyledLine[];
