import { type DOMElement, type DOMNode } from './dom.js';
import type Output from './output.js';
import type { OutputTransformer } from './render-node-to-output.js';
import { StyledLine } from './styled-line.js';
export declare const applyPaddingToStyledChars: (node: DOMElement, lines: StyledLine[]) => StyledLine[];
export declare const calculateWrappedCursorPosition: (lines: StyledLine[], _styledChars: StyledLine, targetOffset: number) => {
    cursorLineIndex: number;
    relativeCursorPosition: number;
};
export declare function handleTextNode(node: DOMElement, output: Output, options: {
    x: number;
    y: number;
    newTransformers: OutputTransformer[];
    selectionMap?: Map<DOMNode, {
        start: number;
        end: number;
    }>;
    selectionStyle?: (line: StyledLine, index: number) => void;
    trackSelection?: boolean;
}): void;
