import { StyledLine } from './styled-line.js';
export declare const CC_BEL: number;
export declare const CC_ESC: number;
export declare const CC_BACKSLASH: number;
export declare const CC_CSI: number;
export declare const CC_OSC: number;
export declare const CC_C1_ST: number;
export declare const CC_0: number;
export declare const CC_9: number;
export declare const CC_SEMI: number;
export declare const CC_M: number;
export declare const ESCAPES: Set<number>;
export declare const linkCodePrefix = "\u001B]8;";
export declare const linkCodePrefixCharCodes: number[];
export declare const linkCodeSuffix = "\u0007";
export declare const linkEndCode = "\u001B]8;;\u0007";
export declare const linkEndCodeST = "\u001B]8;;\u001B\\";
export declare const linkEndCodeC1ST = "\u001B]8;;\u009C";
export declare const endCodesSet: Set<string>;
export declare function getEndCode(code: string): string;
export declare function isIntensityCode(code: AnsiCode): boolean;
export type AnsiCode = {
    type: 'ansi';
    code: string;
    endCode: string;
};
export type ControlCode = {
    type: 'control';
    code: string;
};
export type Char = {
    type: 'char';
    value: string;
    fullWidth: boolean;
};
export type Token = AnsiCode | ControlCode | Char;
export declare function tokenize(str: string, endChar?: number): Token[];
export declare const BOLD_MASK = 1;
export declare const DIM_MASK: number;
export declare const ITALIC_MASK: number;
export declare const UNDERLINE_MASK: number;
export declare const STRIKETHROUGH_MASK: number;
export declare const INVERSE_MASK: number;
export declare const HIDDEN_MASK: number;
export declare const FULL_WIDTH_MASK: number;
export declare function styledLineFromTokens(tokens: Token[]): StyledLine;
export declare function styledLineToString(line: StyledLine): string;
