import { PushResult } from '../../../typings';
import { StringTask } from '../types';
type PushRef = {
    remote?: string;
    branch?: string;
};
export declare function pushTagsTask(ref: PushRef | undefined, customArgs: string[]): StringTask<PushResult>;
export declare function pushTask(ref: PushRef | undefined, customArgs: string[]): StringTask<PushResult>;
export {};
