type ColorType = 'foreground' | 'background';
declare const colorize: (str: string, color: string | undefined, type: ColorType) => string;
export declare const getBackgroundColorEscape: (color: string) => string | undefined;
export default colorize;
