const require = (await import('node:module')).createRequire(import.meta.url); const __chunk_filename = (await import('node:url')).fileURLToPath(import.meta.url); const __chunk_dirname = (await import('node:path')).dirname(__chunk_filename);
import {
  cleanupCheckpoints,
  registerCleanup,
  registerSyncCleanup,
  registerTelemetryConfig,
  removeCleanup,
  removeSyncCleanup,
  resetCleanupForTesting,
  runExitCleanup,
  runSyncCleanup,
  setupSignalHandlers,
  setupTtyCheck
} from "./chunk-UG5REQVB.js";
import "./chunk-7QUQFW7M.js";
import "./chunk-6HI7VNOG.js";
import "./chunk-TUDYL3X4.js";
import "./chunk-IUUIT4SU.js";
import "./chunk-34MYV7JD.js";
export {
  cleanupCheckpoints,
  registerCleanup,
  registerSyncCleanup,
  registerTelemetryConfig,
  removeCleanup,
  removeSyncCleanup,
  resetCleanupForTesting,
  runExitCleanup,
  runSyncCleanup,
  setupSignalHandlers,
  setupTtyCheck
};
