import { RemoteMessageResult, RemoteMessages } from '../../../typings';
import { RemoteLineParser } from '../utils';
export declare const remoteMessagesObjectParsers: RemoteLineParser<RemoteMessageResult<RemoteMessages>>[];
