const require = (await import('node:module')).createRequire(import.meta.url); const __chunk_filename = (await import('node:url')).fileURLToPath(import.meta.url); const __chunk_dirname = (await import('node:path')).dirname(__chunk_filename);
import {
  import_dist,
  init_https_proxy_agent
} from "./chunk-TUDYL3X4.js";
import "./chunk-34MYV7JD.js";
init_https_proxy_agent();
var export_HttpsProxyAgent = import_dist.HttpsProxyAgent;
export {
  export_HttpsProxyAgent as HttpsProxyAgent
};
