;
import * as overview_0 from '/Users/zoltan/Library/Caches/Bit/capsules/05f6140f72f7d4c5be56111dbe63734e37f4e9e4/pnpm.config_env-replace@1.1.0/dist/env-replace.docs.mdx';

export const compositions = [];
export const overview = [overview_0];

export const compositions_metadata = {"compositions":[]};
