/**
 * @license
 * Copyright 2026 Google LLC
 * SPDX-License-Identifier: Apache-2.0
 */
import { type DOMElement, type DOMNode } from './dom.js';
import type Output from './output.js';
import { type StyledLine } from './styled-line.js';
export declare function handleCachedRenderNode(node: DOMElement, output: Output, options: {
    x: number;
    y: number;
    selectionMap?: Map<DOMNode, {
        start: number;
        end: number;
    }>;
    selectionStyle?: (line: StyledLine, index: number) => void;
    trackSelection?: boolean;
}): void;
