import { EventEmitter } from 'node:events';
import process from 'node:process';
import { createContext } from 'react';
/**
`StdinContext` is a React context that exposes the input stream.
*/
const StdinContext = createContext({
    stdin: process.stdin,
    internal_eventEmitter: new EventEmitter(),
    setRawMode() { },
    isRawModeSupported: false,
    internal_exitOnCtrlC: true,
});
StdinContext.displayName = 'InternalStdinContext';
export default StdinContext;
//# sourceMappingURL=StdinContext.js.map