import { type StyledLine } from './styled-line.js';
import { type DOMElement, type DOMNode } from './dom.js';
import Output from './output.js';
export type OutputTransformer = (s: string, index: number) => string;
export declare const renderToStatic: (node: DOMElement, options?: {
    calculateLayout?: boolean;
    skipStaticElements?: boolean;
    isStickyRender?: boolean;
    selectionMap?: Map<DOMNode, {
        start: number;
        end: number;
    }>;
    selectionStyle?: (line: StyledLine, index: number) => void;
    trackSelection?: boolean;
}) => void;
declare function renderNodeToOutput(node: DOMElement, output: Output, options: {
    offsetX?: number;
    offsetY?: number;
    absoluteOffsetX?: number;
    absoluteOffsetY?: number;
    transformers?: OutputTransformer[];
    skipStaticElements: boolean;
    isStickyRender?: boolean;
    skipStickyHeaders?: boolean;
    selectionMap?: Map<DOMNode, {
        start: number;
        end: number;
    }>;
    selectionStyle?: (line: StyledLine, index: number) => void;
    trackSelection?: boolean;
}): void;
export default renderNodeToOutput;
