/**
 * @license
 * Copyright 2026 Google LLC
 * SPDX-License-Identifier: Apache-2.0
 */
export declare const enterSynchronizedOutput = "\u001B[?2026h";
export declare const exitSynchronizedOutput = "\u001B[?2026l";
export declare const resetScrollRegion = "\u001B[r";
export declare const ris = "\u001Bc";
export declare const clearScrollbackStandard = "\u001B[3J";
export declare const homeEraseDown = "\u001B[H\u001B[J";
export declare const getMoveCursorDownCode: (skippedLines: number) => string;
export declare const getMoveCursorUpCode: (skippedLines: number) => string;
export declare const getDeleteLinesCode: (count: number) => string;
export declare const getInsertLinesCode: (count: number) => string;
export declare const getSetScrollRegionCode: (top: number, bottom: number) => string;
