const require = (await import('node:module')).createRequire(import.meta.url); const __chunk_filename = (await import('node:url')).fileURLToPath(import.meta.url); const __chunk_dirname = (await import('node:path')).dirname(__chunk_filename);
import {
  ACTIVATE_SKILL_TOOL_NAME,
  ADMIN_POLICY_TIER,
  AGENT_TOOL_NAME,
  ALLOWED_MCP_SERVER_PRIORITY,
  ALLOWED_TOOLS_FLAG_PRIORITY,
  ALL_BUILTIN_TOOL_NAMES,
  ALL_EDITORS,
  ALWAYS_ALLOW_PRIORITY,
  ALWAYS_ALLOW_PRIORITY_FRACTION,
  ALWAYS_ALLOW_PRIORITY_OFFSET,
  ASK_USER_DISPLAY_NAME,
  ASK_USER_OPTION_PARAM_DESCRIPTION,
  ASK_USER_OPTION_PARAM_LABEL,
  ASK_USER_PARAM_QUESTIONS,
  ASK_USER_QUESTION_PARAM_HEADER,
  ASK_USER_QUESTION_PARAM_MULTI_SELECT,
  ASK_USER_QUESTION_PARAM_OPTIONS,
  ASK_USER_QUESTION_PARAM_PLACEHOLDER,
  ASK_USER_QUESTION_PARAM_QUESTION,
  ASK_USER_QUESTION_PARAM_TYPE,
  ASK_USER_TOOL_NAME,
  AbortError,
  AccountSuspendedError,
  ActionStatus,
  ActivateSkillTool,
  ActivityDetector,
  ActivityMonitor,
  ActivityType,
  AdminControlsSettingsSchema,
  AfterAgentHookOutput,
  AfterModelHookOutput,
  AgentChatHistory,
  AgentExecutionBlockedError,
  AgentExecutionStoppedError,
  AgentLoadError,
  AgentSession,
  AgentTerminateMode,
  ApiErrorEvent,
  ApiKeyUpdatedEvent,
  ApiRequestEvent,
  ApiRequestPhase,
  ApiResponseEvent,
  ApprovalMode,
  AskUserInvocation,
  AskUserTool,
  AudioRecorder,
  AuthProviderType,
  AuthType,
  BASH_HUP_GUARD,
  BINARY_EXTENSIONS,
  BINARY_FILE_PATTERNS,
  BINARY_INJECTION_KEY,
  BadRequestError,
  BaseDeclarativeTool,
  BaseLlmClient,
  BaseToolInvocation,
  BeforeModelHookOutput,
  BeforeToolHookOutput,
  BeforeToolSelectionHookOutput,
  CODE_ASSIST_API_VERSION,
  CODE_ASSIST_ENDPOINT,
  COMMON_DIRECTORY_EXCLUDES,
  COMMON_IGNORE_PATTERNS,
  COMPLETE_TASK_DISPLAY_NAME,
  COMPLETE_TASK_TOOL_NAME,
  CONFIRMATION_REQUIRED_PRIORITY,
  CORE_TOOLS_FLAG_PRIORITY,
  CacheService,
  CanceledError,
  ChangeAuthRequestedError,
  ChatRecordingService,
  ClearcutLogger,
  CloseDiffRequestSchema,
  CodeAssistServer,
  ColorMode,
  CompressionStatus,
  Config,
  ConfigSchema,
  ConfigSource,
  ConsecaPolicyGenerationEvent,
  ConsecaVerdictEvent,
  ConversationFinishedEvent,
  ConversationInteractionInteraction,
  CoreEvent,
  CoreEventEmitter,
  CoreToolCallStatus,
  CreditPurchaseClickEvent,
  CreditsUsedEvent,
  DEFAULT_CONTEXT_FILENAME,
  DEFAULT_CORE_POLICIES_DIR,
  DEFAULT_ENCODING,
  DEFAULT_FAST_ACK_MODEL_CONFIG_KEY,
  DEFAULT_FILE_EXCLUDES,
  DEFAULT_FILE_FILTERING_OPTIONS,
  DEFAULT_GEMINI_EMBEDDING_MODEL,
  DEFAULT_GEMINI_FLASH_LITE_MODEL,
  DEFAULT_GEMINI_FLASH_MODEL,
  DEFAULT_GEMINI_MODEL,
  DEFAULT_GEMINI_MODEL_AUTO,
  DEFAULT_GUI_EDITOR,
  DEFAULT_MAX_ATTEMPTS,
  DEFAULT_MAX_INPUT_CHARS,
  DEFAULT_MAX_LINES_TEXT_FILE,
  DEFAULT_MAX_OUTPUT_CHARS,
  DEFAULT_MAX_TIME_MINUTES,
  DEFAULT_MAX_TURNS,
  DEFAULT_MEMORY_FILE_FILTERING_OPTIONS,
  DEFAULT_MODEL_CONFIGS,
  DEFAULT_OTLP_ENDPOINT,
  DEFAULT_POLICY_TIER,
  DEFAULT_QUERY_STRING,
  DEFAULT_TELEMETRY_TARGET,
  DEFAULT_THINKING_MODE,
  DEFAULT_TOKEN_LIMIT,
  DEFAULT_TRUNCATE_TOOL_OUTPUT_THRESHOLD,
  DISCOVERED_TOOL_PREFIX,
  DOCS_PARAM_PATH,
  DeclarativeTool,
  DefaultHookOutput,
  DiscoveredMCPTool,
  DiscoveredMCPToolInvocation,
  DiscoveredTool,
  EDITOR_DISPLAY_NAMES,
  EDITOR_OPTIONS,
  EDIT_DISPLAY_NAME,
  EDIT_PARAM_ALLOW_MULTIPLE,
  EDIT_PARAM_INSTRUCTION,
  EDIT_PARAM_NEW_STRING,
  EDIT_PARAM_OLD_STRING,
  EDIT_TOOL_NAME,
  EDIT_TOOL_NAMES,
  ENTER_PLAN_MODE_TOOL_NAME,
  EVENT_API_KEY_UPDATED,
  EVENT_CREDITS_USED,
  EVENT_CREDIT_PURCHASE_CLICK,
  EVENT_EMPTY_WALLET_MENU_SHOWN,
  EVENT_OVERAGE_MENU_SHOWN,
  EVENT_OVERAGE_OPTION_SELECTED,
  EXCLUDE_TOOLS_FLAG_PRIORITY,
  EXIT_PLAN_MODE_TOOL_NAME,
  EXIT_PLAN_PARAM_PLAN_FILENAME,
  EXTENSION_POLICY_TIER,
  EditTool,
  EmptyWalletMenuShownEvent,
  EndSessionEvent,
  EventLoopMonitor,
  ExecutionLifecycleService,
  ExitCodes,
  ExperimentFlags,
  ExtensionDisableEvent,
  ExtensionEnableEvent,
  ExtensionInstallEvent,
  ExtensionIntegrityDataSchema,
  ExtensionIntegrityManager,
  ExtensionIntegrityMapSchema,
  ExtensionLoader,
  ExtensionUninstallEvent,
  ExtensionUpdateEvent,
  FETCH_FAILED_MESSAGE,
  FORCE_FILE_STORAGE_ENV_VAR,
  FRONTMATTER_REGEX,
  FakeContentGenerator,
  FatalAuthenticationError,
  FatalCancellationError,
  FatalConfigError,
  FatalError,
  FatalInputError,
  FatalSandboxError,
  FatalToolExecutionError,
  FatalTurnLimitedError,
  FatalUntrustedWorkspaceError,
  FetchAdminControlsResponseSchema,
  FetchError,
  FileDiscoveryService,
  FileExclusions,
  FileOperation,
  FileSchema,
  FileSearchFactory,
  FlashFallbackEvent,
  FolderTrustDiscoveryService,
  ForbiddenError,
  G1_CREDIT_TYPE,
  G1_UTM_CAMPAIGNS,
  GEMINI_CLI_COMPANION_EXTENSION_NAME,
  GEMINI_CLI_IDENTIFICATION_ENV_VAR,
  GEMINI_CLI_IDENTIFICATION_ENV_VAR_VALUE,
  GEMINI_DIR,
  GEMINI_IGNORE_FILE_NAME,
  GEMINI_MODEL_ALIAS_AUTO,
  GEMINI_MODEL_ALIAS_FLASH,
  GEMINI_MODEL_ALIAS_FLASH_LITE,
  GEMINI_MODEL_ALIAS_PRO,
  GEMMA_4_26B_A4B_IT_MODEL,
  GEMMA_4_31B_IT_MODEL,
  GEMMA_4_TOKEN_LIMIT,
  GEN_AI_AGENT_DESCRIPTION,
  GEN_AI_AGENT_NAME,
  GEN_AI_CONVERSATION_ID,
  GEN_AI_INPUT_MESSAGES,
  GEN_AI_OPERATION_NAME,
  GEN_AI_OUTPUT_MESSAGES,
  GEN_AI_PROMPT_NAME,
  GEN_AI_REQUEST_MODEL,
  GEN_AI_RESPONSE_MODEL,
  GEN_AI_SYSTEM_INSTRUCTIONS,
  GEN_AI_TOOL_CALL_ID,
  GEN_AI_TOOL_DEFINITIONS,
  GEN_AI_TOOL_DESCRIPTION,
  GEN_AI_TOOL_NAME,
  GEN_AI_USAGE_INPUT_TOKENS,
  GEN_AI_USAGE_OUTPUT_TOKENS,
  GET_INTERNAL_DOCS_TOOL_NAME,
  GLOB_DISPLAY_NAME,
  GLOB_TOOL_NAME,
  GOOGLE_ACCOUNTS_FILENAME,
  GOVERNANCE_FILES,
  GREP_DISPLAY_NAME,
  GREP_PARAM_AFTER,
  GREP_PARAM_BEFORE,
  GREP_PARAM_CONTEXT,
  GREP_PARAM_EXCLUDE_PATTERN,
  GREP_PARAM_FIXED_STRINGS,
  GREP_PARAM_INCLUDE_PATTERN,
  GREP_PARAM_MAX_MATCHES_PER_FILE,
  GREP_PARAM_NAMES_ONLY,
  GREP_PARAM_NO_IGNORE,
  GREP_PARAM_TOTAL_MAX_MATCHES,
  GREP_TOOL_NAME,
  GcpLogExporter,
  GcpMetricExporter,
  GcpTraceExporter,
  GeminiChat,
  GeminiCliOperation,
  GeminiClient,
  GeminiEventType,
  GeminiLiveTranscriptionProvider,
  GenAiOperationName,
  GenAiProviderName,
  GenAiTokenType,
  GitIgnoreParser,
  GitService,
  GlobTool,
  GrepTool,
  HOOKS_CONFIG_FIELDS,
  HighWaterMarkTracker,
  HookAggregator,
  HookEventHandler,
  HookEventName,
  HookPlanner,
  HookRegistry,
  HookRunner,
  HookSystem,
  HookType,
  IDEConnectionStatus,
  IDE_DEFINITIONS,
  IDE_MAX_OPEN_FILES,
  IDE_MAX_SELECTED_TEXT_LENGTH,
  IDE_REQUEST_TIMEOUT_MS,
  INCOMPLETE_JSON_MESSAGE,
  INITIAL_HISTORY_LENGTH,
  INTEGRITY_FILENAME,
  INTEGRITY_KEY_FILENAME,
  IdeClient,
  IdeConnectionEvent,
  IdeConnectionType,
  IdeContextNotificationSchema,
  IdeContextSchema,
  IdeContextStore,
  IdeDiffAcceptedNotificationSchema,
  IdeDiffClosedNotificationSchema,
  IdeDiffRejectedNotificationSchema,
  InProcessCheckerType,
  IneligibleTierError,
  IneligibleTierReasonCode,
  InitiationMethod,
  InjectionService,
  IntegrityDataStatus,
  IntegrityStatus,
  IntegrityStoreSchema,
  InvalidNumericProjectIdError,
  InvalidStreamError,
  JsonFormatter,
  JsonStreamEventType,
  KEYCHAIN_SERVICE_NAME,
  KEYCHAIN_TEST_PREFIX,
  KeychainSchema,
  KeychainService,
  KeychainTokenStorage,
  Kind,
  LIST_MCP_RESOURCES_TOOL_NAME,
  LIVE_OUTPUT_MAX_BUFFER_CHARS,
  LSTool,
  LS_DISPLAY_NAME,
  LS_PARAM_IGNORE,
  LS_TOOL_NAME,
  LegacyAgentProtocol,
  LegacyAgentSession,
  LlmRole,
  LoadedTrustedFolders,
  LocalAgentExecutor,
  LocalSandboxManager,
  Logger,
  LoggingContentGenerator,
  MAX_FILE_SIZE_MB,
  MAX_HISTORY_MESSAGES,
  MAX_LINE_LENGTH_TEXT_FILE,
  MAX_TOOL_OUTPUT_SIZE,
  MCPDiscoveryState,
  MCPOAuthProvider,
  MCPOAuthTokenStorage,
  MCPServerConfig,
  MCPServerStatus,
  MCP_DEFAULT_TIMEOUT_MSEC,
  MCP_EXCLUDED_PRIORITY,
  MCP_QUALIFIED_NAME_SEPARATOR,
  MCP_TOOL_PREFIX,
  MEDIA_FILE_PATTERNS,
  MIN_CREDIT_BALANCE,
  MODES_BY_PERMISSIVENESS,
  MUTATOR_KINDS,
  McpClient,
  McpConfigDefinitionSchema,
  MemoryContextManager,
  MemoryMetricType,
  MemoryMonitor,
  MessageBus,
  MessageBusType,
  MessageSenderType,
  ModelConfigService,
  ModelNotFoundError,
  ModelRouterService,
  ModelSlashCommandEvent,
  NO_EDITOR_AVAILABLE_ERROR,
  NetworkRetryAttemptEvent,
  NodeType,
  NoopSandboxManager,
  NotificationType,
  OAuthUtils,
  OUTPUT_UPDATE_INTERVAL_MS,
  OVERAGE_ELIGIBLE_MODELS,
  OnboardUserStatusCode,
  OnboardingStartEvent,
  OnboardingSuccessEvent,
  OpenDiffRequestSchema,
  OutputFormat,
  OverageMenuShownEvent,
  OverageOptionSelectedEvent,
  PARAM_CASE_SENSITIVE,
  PARAM_DESCRIPTION,
  PARAM_DIR_PATH,
  PARAM_FILE_FILTERING_OPTIONS,
  PARAM_FILE_PATH,
  PARAM_PATTERN,
  PARAM_RESPECT_GEMINI_IGNORE,
  PARAM_RESPECT_GIT_IGNORE,
  PLAN_MODE_PARAM_REASON,
  PLAN_MODE_TOOLS,
  PREVIEW_GEMINI_3_1_CUSTOM_TOOLS_MODEL,
  PREVIEW_GEMINI_3_1_FLASH_LITE_MODEL,
  PREVIEW_GEMINI_3_1_MODEL,
  PREVIEW_GEMINI_FLASH_MODEL,
  PREVIEW_GEMINI_MODEL,
  PREVIEW_GEMINI_MODEL_AUTO,
  PRIORITY_SUBAGENT_TOOL,
  PRIORITY_YOLO_ALLOW_ALL,
  PROJECT_MEMORY_INDEX_FILENAME,
  PYTHON_EXCLUDES,
  PerformanceMetricType,
  PlanErrorMessages,
  PolicyDecision,
  PolicyEngine,
  PolicyIntegrityManager,
  PreCompressTrigger,
  PrivateIpError,
  ProjectIdRequiredError,
  QuestionType,
  READ_FILE_DISPLAY_NAME,
  READ_FILE_PARAM_END_LINE,
  READ_FILE_PARAM_START_LINE,
  READ_FILE_TOOL_NAME,
  READ_MANY_FILES_DISPLAY_NAME,
  READ_MANY_FILES_TOOL_NAME,
  READ_MANY_PARAM_EXCLUDE,
  READ_MANY_PARAM_INCLUDE,
  READ_MANY_PARAM_RECURSIVE,
  READ_MANY_PARAM_USE_DEFAULT_EXCLUDES,
  READ_MCP_RESOURCE_TOOL_NAME,
  READ_ONLY_KINDS,
  REDIRECTION_NAMES,
  REFERENCE_CONTENT_END,
  REFERENCE_CONTENT_START,
  RELEASE_CHANNEL_STABILITY,
  ROOT_SCHEDULER_ID,
  RateLimiter,
  ReadFileTool,
  ReadManyFilesTool,
  RecordingContentGenerator,
  ReleaseChannel,
  RequiredMcpServerConfigSchema,
  ResourceRegistry,
  RetryableQuotaError,
  RewindEvent,
  RipGrepTool,
  SCROLLBACK_LIMIT,
  SECRET_FILES,
  SECRET_KEY_ACCOUNT,
  SECRET_PREFIX,
  SERVICE_DESCRIPTION,
  SERVICE_NAME,
  SESSION_FILE_PREFIX,
  SHADOW_REPO_AUTHOR_EMAIL,
  SHADOW_REPO_AUTHOR_NAME,
  SHELL_PARAM_COMMAND,
  SHELL_PARAM_IS_BACKGROUND,
  SHELL_TOOL_NAME,
  SHELL_TOOL_NAMES,
  SKILL_PARAM_NAME,
  SUBAGENT_CANCELLED_ERROR_MESSAGE,
  SUBAGENT_REJECTED_ERROR_PREFIX,
  SYNTHETIC_THOUGHT_SIGNATURE,
  SYSTEM_FILE_EXCLUDES,
  SandboxedFileSystemService,
  Scheduler,
  SchemaValidator,
  SemanticAttributes,
  SessionEndReason,
  SessionStartSource,
  ShellExecutionService,
  ShellTool,
  ShellToolInvocation,
  SimpleExtensionLoader,
  SkillManager,
  SlashCommandStatus,
  SnapshotGenerator,
  StandardFileSystemService,
  StartSessionEvent,
  StartupProfiler,
  Storage,
  StreamEventType,
  StreamJsonFormatter,
  SubagentActivityErrorType,
  SubagentState,
  TASK_TYPE_LABELS,
  TODOS_ITEM_PARAM_DESCRIPTION,
  TODOS_ITEM_PARAM_STATUS,
  TODOS_PARAM_TODOS,
  TOOLS_REQUIRING_NARROWING,
  TOOL_LEGACY_ALIASES,
  TOOL_OUTPUTS_DIR,
  TOPIC_PARAM_STRATEGIC_INTENT,
  TOPIC_PARAM_SUMMARY,
  TOPIC_PARAM_TITLE,
  TRACKER_ADD_DEPENDENCY_TOOL_NAME,
  TRACKER_CREATE_TASK_TOOL_NAME,
  TRACKER_GET_TASK_TOOL_NAME,
  TRACKER_LIST_TASKS_TOOL_NAME,
  TRACKER_UPDATE_TASK_TOOL_NAME,
  TRACKER_VISUALIZE_TOOL_NAME,
  TRUSTED_FOLDERS_FILENAME,
  TRUSTED_MCP_SERVER_PRIORITY,
  TaskStatus,
  TaskStatusSchema,
  TaskType,
  TaskTypeSchema,
  TelemetryTarget,
  TerminalQuotaError,
  ToolCallDecision,
  ToolCallEvent,
  ToolConfirmationOutcome,
  ToolErrorType,
  ToolExecutionPhase,
  ToolExecutor,
  ToolOutputTruncatedEvent,
  ToolRegistry,
  TrackerAddDependencyTool,
  TrackerCreateTaskTool,
  TrackerGetTaskTool,
  TrackerListTasksTool,
  TrackerService,
  TrackerTaskSchema,
  TrackerUpdateTaskTool,
  TrackerVisualizeTool,
  TranscriptionFactory,
  TrustLevel,
  Turn,
  UPDATE_TOPIC_DISPLAY_NAME,
  UPDATE_TOPIC_TOOL_NAME,
  UPGRADE_URL_PAGE,
  USER_POLICY_TIER,
  USER_STEERING_INSTRUCTION,
  UiTelemetryService,
  UnauthorizedError,
  UserAccountManager,
  UserPromptEvent,
  UserTierId,
  VALID_GEMINI_MODELS,
  ValidationCancelledError,
  ValidationRequiredError,
  WEB_FETCH_DISPLAY_NAME,
  WEB_FETCH_PARAM_PROMPT,
  WEB_FETCH_TOOL_NAME,
  WEB_SEARCH_DISPLAY_NAME,
  WEB_SEARCH_PARAM_QUERY,
  WEB_SEARCH_TOOL_NAME,
  WORKSPACE_POLICY_TIER,
  WRITE_FILE_DISPLAY_NAME,
  WRITE_FILE_PARAM_CONTENT,
  WRITE_FILE_TOOL_NAME,
  WRITE_TODOS_TOOL_NAME,
  WarningPriority,
  WebFetchFallbackAttemptEvent,
  WebFetchTool,
  WebSearchTool,
  WhisperModelManager,
  WhisperTranscriptionProvider,
  WindowsSandboxManager,
  WorkspaceContext,
  WorktreeService,
  WriteFileTool,
  WriteTodosTool,
  _clearCache,
  addMCPStatusChangeListener,
  allowEditorTypeInSandbox,
  appendToLastTextPart,
  applyAdminAllowlist,
  applyInboxMemoryPatch,
  applyInboxPatch,
  applyReplacement,
  applyRequiredServers,
  assertValidPathString,
  assumeExhaustive,
  authEvents,
  belongsInConfirmationQueue,
  buildG1Url,
  buildTodosReturnDisplay,
  buildToolResponseData,
  buildToolVisibilityContext,
  buildUserSteeringHintPrompt,
  bytesToMB,
  calculateReplacement,
  captureHeapSnapshot,
  categorizeAndConcatenate,
  checkExhaustive,
  checkPathTrust,
  checkPolicy,
  classifyGoogleError,
  cleanupWorktree,
  clearApiKey,
  clearCachedCredentialFile,
  clearEmittedPolicyWarnings,
  clearOauthClientCache,
  clearRealPathCacheForTesting,
  cloneFunctionCall,
  cloneFunctionResponse,
  clonePart,
  computeModelAddedAndRemovedLines,
  concatenateInstructions,
  connectAndDiscover,
  connectToMcpServer,
  contentPartsToGeminiParts,
  convertColorToHex,
  convertGithubUrlToRaw,
  convertSessionToClientHistory,
  convertToFunctionResponse,
  convertToRestPayload,
  coreEvents,
  createCache,
  createCodeAssistContentGenerator,
  createContentGenerator,
  createContentGeneratorConfig,
  createConversationOffered,
  createHookOutput,
  createPolicyEngineConfig,
  createPolicyUpdater,
  createSafeProxyAgent,
  createSandboxManager,
  createSessionId,
  createTranslationState,
  createTransport,
  createUnauthorizedToolError,
  createWorkingStdio,
  createWorktree,
  createWorktreeService,
  debugLogger,
  decodeTagName,
  deduplicateAbsolutePaths,
  deduplicatePathsByFileIdentity,
  deleteSessionArtifactsAsync,
  deleteSessionFileAndArtifacts,
  deleteStoredSession,
  deleteSubagentSessionDirAndArtifactsAsync,
  deriveSessionShortId,
  detectBOM,
  detectCommandSubstitution,
  detectFileType,
  detectIdeFromEnv,
  detectLineEnding,
  disableBracketedPasteMode,
  disableKittyKeyboardProtocol,
  disableLineWrapping,
  disableModifyOtherKeys,
  disableMouseEvents,
  disableMouseTracking,
  discoverMcpTools,
  discoverPrompts,
  discoverResources,
  discoverTools,
  dismissInboxMemoryPatch,
  dismissInboxPatch,
  dismissInboxSkill,
  displayContentToString,
  doesToolInvocationMatch,
  enableBracketedPasteMode,
  enableKittyKeyboardProtocol,
  enableLineWrapping,
  enableModifyOtherKeys,
  enableMouseEvents,
  encodeTagName,
  ensureStableToolIds,
  enterAlternateScreen,
  escapePath,
  escapeShellArg,
  estimateContextBreakdown,
  execStreaming,
  exitAlternateScreen,
  extractExtensionsFromPatterns,
  extractStringFromParseEntry,
  fetchAdminControls,
  fetchAdminControlsOnce,
  fetchWithTimeout,
  fileExists,
  filter,
  findGitRoot,
  findSecretFiles,
  flatMapTextParts,
  flattenMemory,
  flushTelemetry,
  formatBackgroundCompletionForModel,
  formatBytes,
  formatCheckpointDisplayList,
  formatForSpeech,
  formatMcpToolName,
  formatPolicyError,
  formatProtoJsonDuration,
  formatTruncatedToolOutput,
  formatUserHintsForModel,
  geminiPartsToContentParts,
  generalistProfile,
  generalistProfile2,
  generateCheckpointFileName,
  generateFastAckText,
  generateSteeringAckMessage,
  generateSummary,
  generateValidName,
  getAbsoluteGitDir,
  getActivityDetector,
  getActivityMonitor,
  getAdminBlockedMcpServersMessage,
  getAdminErrorMessage,
  getAgentCardLoadOptions,
  getAllGeminiMdFilenames,
  getAllMCPServerStatuses,
  getAllowedMemoryPatchRoots,
  getAlwaysAllowPriorityFraction,
  getApprovalModeDescription,
  getAuthTypeFromEnv,
  getAutoModelDescription,
  getAvailablePort,
  getChannelFromVersion,
  getCheckpointInfoList,
  getCitations,
  getCodeAssistServer,
  getCommandName,
  getCommandRoot,
  getCommandRoots,
  getCompatibilityWarnings,
  getCompressionPrompt,
  getConsentForOauth,
  getConventionAttributes,
  getCoreSystemPrompt,
  getCorrectedFileContent,
  getCurrentGeminiMdFilename,
  getDiffCommand,
  getDirectoryContextString,
  getDisplayString,
  getEditorCommand,
  getEditorDisplayName,
  getEditorExtraArgs,
  getEditorWaitFlag,
  getEnvironmentContext,
  getEnvironmentMemoryPaths,
  getErrorMessage,
  getErrorReplaceResult,
  getErrorStatus,
  getErrorType,
  getEventLoopMonitor,
  getExperiments,
  getExtensionMemoryPaths,
  getFileDiffFromResultDisplay,
  getFolderStructure,
  getFsErrorMessage,
  getFunctionCalls,
  getFunctionCallsAsJson,
  getFunctionCallsFromParts,
  getFunctionCallsFromPartsAsJson,
  getG1CreditBalance,
  getGlobalMemoryFilePath,
  getGlobalMemoryPaths,
  getHookKey,
  getHookSource,
  getIdeInstaller,
  getInitialChatHistory,
  getMCPDiscoveryState,
  getMCPServerPrompts,
  getMCPServerStatus,
  getMatchingSessionFiles,
  getMemoryMonitor,
  getOauthClient,
  getPackageJson,
  getPlanModeExitMessage,
  getPolicyDenialError,
  getPolicyDirectories,
  getPolicyTier,
  getPreviousSession,
  getProjectHash,
  getProjectMemoryIndexFilePath,
  getProjectRootForWorktree,
  getPromptIdWithFallback,
  getPty,
  getRealPath,
  getReleaseChannel,
  getRemoteAgentTargetUrl,
  getResponseText,
  getResponseTextFromParts,
  getRetryErrorType,
  getSecretFileFindArgs,
  getShellConfiguration,
  getSpecificMimeType,
  getStructuredResponse,
  getStructuredResponseFromParts,
  getToolAliases,
  getToolCallDataSchema,
  getToolSuggestion,
  getTruncatedCheckpointNames,
  getUserProjectMemoryPaths,
  getVersion,
  getWorktreePath,
  handleFallback,
  hasCycleInSchema,
  hasNetworkTransport,
  hasRedirection,
  hasSummary,
  hasValidEditorCommand,
  hasValidEditorCommandAsync,
  hasWorktreeChanges,
  homedir,
  ideContextStore,
  initializeActivityMonitor,
  initializeMemoryMonitor,
  initializeShellParsers,
  initializeTelemetry,
  invokeMcpPrompt,
  isAbortError,
  isAccountSuspendedError,
  isActiveModel,
  isAddressPrivate,
  isAgentThought,
  isAgentYield,
  isApiError,
  isAppleTerminal,
  isAuthenticationError,
  isAutoModel,
  isBackgroundExecutionData,
  isBinary,
  isBinaryAvailable,
  isBinaryFile,
  isCloudShell,
  isCompletedAskUserTool,
  isCustomModel,
  isDumbTerminal,
  isEditToolParams,
  isEditorAvailable,
  isEditorAvailableAsync,
  isEmpty,
  isEnabled,
  isFatalToolError,
  isFileDiff,
  isGemini2Model,
  isGemini3Model,
  isGeminiWorktree,
  isGitRepository,
  isGnuScreen,
  isGrepResult,
  isGuiEditor,
  isHeadlessMode,
  isInvalidArgumentError,
  isJetBrainsTerminal,
  isListResult,
  isLoopbackHost,
  isLowColorTmux,
  isMaskedTool,
  isMcpToolAnnotation,
  isMcpToolName,
  isNightly,
  isNodeError,
  isOverageEligibleModel,
  isPerformanceMonitoringActive,
  isPreview,
  isPreviewModel,
  isPrivateIp,
  isPrivateIpAsync,
  isProModel,
  isProjectSkillPatchTarget,
  isReadManyFilesResult,
  isRecord,
  isRenderedInHistory,
  isRetryableError,
  isRollingSummary,
  isSchemaDepthError,
  isSecretFile,
  isSnapshot,
  isStable,
  isStructuredError,
  isStructuredToolResult,
  isSubagentProgress,
  isSubpath,
  isSystemEvent,
  isTelemetrySdkInitialized,
  isTerminalEditor,
  isTextPart,
  isTmux,
  isTool,
  isToolActivityError,
  isToolCallResponseInfo,
  isToolExecution,
  isTrustLevel,
  isTrustedSystemPath,
  isUserActive,
  isUserPrompt,
  isUserVisibleHook,
  isValidEditorType,
  isValidNonThoughtTextPart,
  isValidPathString,
  isValidToolName,
  isVisibleInToolGroup,
  isWindows,
  isWindows10,
  isWithinRoot,
  isWriteFileToolParams,
  jsonToMarkdown,
  listExtensions,
  listInboxMemoryPatches,
  listInboxPatches,
  listInboxSkills,
  listMemoryFiles,
  loadAgentsFromDirectory,
  loadApiKey,
  loadConversationRecord,
  loadExtensionPolicies,
  loadJitSubdirectoryMemory,
  loadPoliciesFromToml,
  loadSkillFromFile,
  loadSkillsFromDir,
  loadTrustedFolders,
  loadWasmBinary,
  logApiError,
  logApiRequest,
  logApiResponse,
  logBillingEvent,
  logChatCompression,
  logCliConfiguration,
  logConsecaPolicyGeneration,
  logConsecaVerdict,
  logConversationFinishedEvent,
  logExtensionDisable,
  logExtensionEnable,
  logExtensionInstallEvent,
  logExtensionUninstall,
  logExtensionUpdateEvent,
  logFlashFallback,
  logIdeConnection,
  logModelSlashCommand,
  logNetworkRetryAttempt,
  logOnboardingStart,
  logOnboardingSuccess,
  logRewind,
  logSlashCommand,
  logToolCall,
  logToolOutputTruncated,
  logUserPrompt,
  logWebFetchFallbackAttempt,
  makeChatCompressionEvent,
  makeFakeConfig,
  makeRelative,
  makeSlashCommandEvent,
  mapError,
  mapFinishReason,
  mapHttpToGrpcStatus,
  mapUsage,
  markdownToAgentDefinition,
  mcpServerRequiresOAuth,
  moveInboxSkill,
  normalizeCommand,
  normalizePath,
  normalizeSpace,
  normalizeUrl,
  openBrowserSecurely,
  openDiff,
  parseAgentMarkdown,
  parseAndFormatApiError,
  parseBashCommandDetails,
  parseBooleanEnvFlag,
  parseCommandDetails,
  parseFrontmatter,
  parseGoogleApiError,
  parseMcpToolName,
  parsePrompt,
  parseTelemetryTargetValue,
  parseThought,
  partListUnionToString,
  partToString,
  patchStdio,
  performInit,
  performRestore,
  populateMcpServerCommand,
  populateToolDisplay,
  processRestorableToolCalls,
  processSingleFileContent,
  promptIdContext,
  readFileWithEncoding,
  readGeminiMdFiles,
  readPathFromWorkspace,
  readPolicyFiles,
  readWasmBinaryFromDisk,
  recordApiErrorMetrics,
  recordApiRequestBreakdown,
  recordApiResponseMetrics,
  recordBaselineComparison,
  recordContentRetry,
  recordContentRetryFailure,
  recordConversationOffered,
  recordCpuUsage,
  recordCreditPurchaseClick,
  recordCurrentMemoryUsage,
  recordCustomApiResponseMetrics,
  recordCustomTokenUsageMetrics,
  recordEventLoopDelay,
  recordExitFail,
  recordFileOperationMetric,
  recordFlickerFrame,
  recordGenAiClientOperationDuration,
  recordGenAiClientTokenUsage,
  recordInvalidChunk,
  recordMemoryUsage,
  recordModelRoutingMetrics,
  recordOverageOptionSelected,
  recordPerformanceRegression,
  recordPerformanceScore,
  recordRetryAttemptMetrics,
  recordSlowRender,
  recordStartupPerformance,
  recordTokenEfficiency,
  recordTokenUsageMetrics,
  recordToolCallInteractions,
  recordToolCallMetrics,
  recordToolExecutionBreakdown,
  recordToolQueueDepth,
  recordUserActivity,
  refreshMemory,
  removeMCPStatusChangeListener,
  renderDisplayDiff,
  resetApiKeyCacheForTesting,
  resetBrowserSession,
  resetGeminiMdFilename,
  resetOauthClientForTesting,
  resetTrustedFoldersForTesting,
  resetUserDataCacheForTesting,
  resetVersionCache,
  resolveAndValidatePlanPath,
  resolveAtCommandPath,
  resolveClassifierModel,
  resolveEditorAsync,
  resolveEditorTypeFromCommand,
  resolveExecutable,
  resolveModel,
  resolvePathFromEnv,
  resolveRipgrepPath,
  resolveSandboxPaths,
  resolveTelemetrySettings,
  resolveToRealPath,
  retryWithBackoff,
  runInDevTraceSpan,
  safeJsonToMarkdown,
  safeLiteralReplace,
  safeTemplateReplace,
  sanitizeAdminSettings,
  sanitizeFilenamePart,
  sanitizeHostname,
  sanitizeOutput,
  saveApiKey,
  saveTruncatedToolOutput,
  saveTrustedFolders,
  scheduleAgentTools,
  serializeTerminalToObject,
  setGeminiMdFilename,
  setGlobalProxy,
  setupUser,
  shortenPath,
  shouldAttemptBrowserLaunch,
  shouldAutoUseCredits,
  shouldEnterAlternateScreen,
  shouldLaunchBrowser,
  shouldShowEmptyWalletMenu,
  shouldShowOverageMenu,
  showMemory,
  shutdownTelemetry,
  sortFileEntries,
  spawnAsync,
  splitCommands,
  startGlobalActivityMonitoring,
  startGlobalEventLoopMonitoring,
  startGlobalMemoryMonitoring,
  startMemoryService,
  startupProfiler,
  stopAdminControlsPolling,
  stopGlobalActivityMonitoring,
  stopGlobalEventLoopMonitoring,
  stopGlobalMemoryMonitoring,
  stressTestProfile,
  stripAnsiFromBuffer,
  stripHupGuard,
  stripShellWrapper,
  stripToolCallIdPrefixes,
  supports256Colors,
  supportsModernFeatures,
  supportsMultimodalFunctionResponse,
  supportsTrueColor,
  tildeifyPath,
  tmpdir,
  toAbsolutePath,
  toFriendlyError,
  toPathKey,
  tokenLimit,
  toolResultDisplayToDisplayContent,
  translateEvent,
  truncateFastAckInput,
  truncateString,
  uiTelemetryService,
  unescapePath,
  updateGlobalFetchTimeouts,
  updateMCPServerStatus,
  updatePart,
  updatePolicy,
  validateAndSanitizeSessionId,
  validateMcpPolicyToolNames,
  validatePatches,
  validatePath,
  validatePlanContent,
  validatePlanPath,
  wrapInAccountChooser,
  writeToStderr,
  writeToStdout
} from "./chunk-7QUQFW7M.js";
import "./chunk-6HI7VNOG.js";
import "./chunk-TUDYL3X4.js";
import {
  SpanStatusCode,
  ValueType
} from "./chunk-IUUIT4SU.js";
import "./chunk-34MYV7JD.js";
export {
  ACTIVATE_SKILL_TOOL_NAME,
  ADMIN_POLICY_TIER,
  AGENT_TOOL_NAME,
  ALLOWED_MCP_SERVER_PRIORITY,
  ALLOWED_TOOLS_FLAG_PRIORITY,
  ALL_BUILTIN_TOOL_NAMES,
  ALL_EDITORS,
  ALWAYS_ALLOW_PRIORITY,
  ALWAYS_ALLOW_PRIORITY_FRACTION,
  ALWAYS_ALLOW_PRIORITY_OFFSET,
  ASK_USER_DISPLAY_NAME,
  ASK_USER_OPTION_PARAM_DESCRIPTION,
  ASK_USER_OPTION_PARAM_LABEL,
  ASK_USER_PARAM_QUESTIONS,
  ASK_USER_QUESTION_PARAM_HEADER,
  ASK_USER_QUESTION_PARAM_MULTI_SELECT,
  ASK_USER_QUESTION_PARAM_OPTIONS,
  ASK_USER_QUESTION_PARAM_PLACEHOLDER,
  ASK_USER_QUESTION_PARAM_QUESTION,
  ASK_USER_QUESTION_PARAM_TYPE,
  ASK_USER_TOOL_NAME,
  AbortError,
  AccountSuspendedError,
  ActionStatus,
  ActivateSkillTool,
  ActivityDetector,
  ActivityMonitor,
  ActivityType,
  AdminControlsSettingsSchema,
  AfterAgentHookOutput,
  AfterModelHookOutput,
  AgentChatHistory,
  AgentExecutionBlockedError,
  AgentExecutionStoppedError,
  AgentLoadError,
  AgentSession,
  AgentTerminateMode,
  ApiErrorEvent,
  ApiKeyUpdatedEvent,
  ApiRequestEvent,
  ApiRequestPhase,
  ApiResponseEvent,
  ApprovalMode,
  AskUserInvocation,
  AskUserTool,
  AudioRecorder,
  AuthProviderType,
  AuthType,
  BASH_HUP_GUARD,
  BINARY_EXTENSIONS,
  BINARY_FILE_PATTERNS,
  BINARY_INJECTION_KEY,
  BadRequestError,
  BaseDeclarativeTool,
  BaseLlmClient,
  BaseToolInvocation,
  BeforeModelHookOutput,
  BeforeToolHookOutput,
  BeforeToolSelectionHookOutput,
  CODE_ASSIST_API_VERSION,
  CODE_ASSIST_ENDPOINT,
  COMMON_DIRECTORY_EXCLUDES,
  COMMON_IGNORE_PATTERNS,
  COMPLETE_TASK_DISPLAY_NAME,
  COMPLETE_TASK_TOOL_NAME,
  CONFIRMATION_REQUIRED_PRIORITY,
  CORE_TOOLS_FLAG_PRIORITY,
  CacheService,
  CanceledError,
  ChangeAuthRequestedError,
  ChatRecordingService,
  ClearcutLogger,
  CloseDiffRequestSchema,
  CodeAssistServer,
  ColorMode,
  CompressionStatus,
  Config,
  ConfigSchema,
  ConfigSource,
  ConsecaPolicyGenerationEvent,
  ConsecaVerdictEvent,
  ConversationFinishedEvent,
  ConversationInteractionInteraction,
  CoreEvent,
  CoreEventEmitter,
  CoreToolCallStatus,
  CreditPurchaseClickEvent,
  CreditsUsedEvent,
  DEFAULT_CONTEXT_FILENAME,
  DEFAULT_CORE_POLICIES_DIR,
  DEFAULT_ENCODING,
  DEFAULT_FAST_ACK_MODEL_CONFIG_KEY,
  DEFAULT_FILE_EXCLUDES,
  DEFAULT_FILE_FILTERING_OPTIONS,
  DEFAULT_GEMINI_EMBEDDING_MODEL,
  DEFAULT_GEMINI_FLASH_LITE_MODEL,
  DEFAULT_GEMINI_FLASH_MODEL,
  DEFAULT_GEMINI_MODEL,
  DEFAULT_GEMINI_MODEL_AUTO,
  DEFAULT_GUI_EDITOR,
  DEFAULT_MAX_ATTEMPTS,
  DEFAULT_MAX_INPUT_CHARS,
  DEFAULT_MAX_LINES_TEXT_FILE,
  DEFAULT_MAX_OUTPUT_CHARS,
  DEFAULT_MAX_TIME_MINUTES,
  DEFAULT_MAX_TURNS,
  DEFAULT_MEMORY_FILE_FILTERING_OPTIONS,
  DEFAULT_MODEL_CONFIGS,
  DEFAULT_OTLP_ENDPOINT,
  DEFAULT_POLICY_TIER,
  DEFAULT_QUERY_STRING,
  DEFAULT_TELEMETRY_TARGET,
  DEFAULT_THINKING_MODE,
  DEFAULT_TOKEN_LIMIT,
  DEFAULT_TRUNCATE_TOOL_OUTPUT_THRESHOLD,
  DISCOVERED_TOOL_PREFIX,
  DOCS_PARAM_PATH,
  DeclarativeTool,
  DefaultHookOutput,
  DiscoveredMCPTool,
  DiscoveredMCPToolInvocation,
  DiscoveredTool,
  EDITOR_DISPLAY_NAMES,
  EDITOR_OPTIONS,
  EDIT_DISPLAY_NAME,
  EDIT_PARAM_ALLOW_MULTIPLE,
  EDIT_PARAM_INSTRUCTION,
  EDIT_PARAM_NEW_STRING,
  EDIT_PARAM_OLD_STRING,
  EDIT_TOOL_NAME,
  EDIT_TOOL_NAMES,
  ENTER_PLAN_MODE_TOOL_NAME,
  EVENT_API_KEY_UPDATED,
  EVENT_CREDITS_USED,
  EVENT_CREDIT_PURCHASE_CLICK,
  EVENT_EMPTY_WALLET_MENU_SHOWN,
  EVENT_OVERAGE_MENU_SHOWN,
  EVENT_OVERAGE_OPTION_SELECTED,
  EXCLUDE_TOOLS_FLAG_PRIORITY,
  EXIT_PLAN_MODE_TOOL_NAME,
  EXIT_PLAN_PARAM_PLAN_FILENAME,
  EXTENSION_POLICY_TIER,
  EditTool,
  EmptyWalletMenuShownEvent,
  EndSessionEvent,
  EventLoopMonitor,
  ExecutionLifecycleService,
  ExitCodes,
  ExperimentFlags,
  ExtensionDisableEvent,
  ExtensionEnableEvent,
  ExtensionInstallEvent,
  ExtensionIntegrityDataSchema,
  ExtensionIntegrityManager,
  ExtensionIntegrityMapSchema,
  ExtensionLoader,
  ExtensionUninstallEvent,
  ExtensionUpdateEvent,
  FETCH_FAILED_MESSAGE,
  FORCE_FILE_STORAGE_ENV_VAR,
  FRONTMATTER_REGEX,
  FakeContentGenerator,
  FatalAuthenticationError,
  FatalCancellationError,
  FatalConfigError,
  FatalError,
  FatalInputError,
  FatalSandboxError,
  FatalToolExecutionError,
  FatalTurnLimitedError,
  FatalUntrustedWorkspaceError,
  FetchAdminControlsResponseSchema,
  FetchError,
  FileDiscoveryService,
  FileExclusions,
  FileOperation,
  FileSchema,
  FileSearchFactory,
  FlashFallbackEvent,
  FolderTrustDiscoveryService,
  ForbiddenError,
  G1_CREDIT_TYPE,
  G1_UTM_CAMPAIGNS,
  GEMINI_CLI_COMPANION_EXTENSION_NAME,
  GEMINI_CLI_IDENTIFICATION_ENV_VAR,
  GEMINI_CLI_IDENTIFICATION_ENV_VAR_VALUE,
  GEMINI_DIR,
  GEMINI_IGNORE_FILE_NAME,
  GEMINI_MODEL_ALIAS_AUTO,
  GEMINI_MODEL_ALIAS_FLASH,
  GEMINI_MODEL_ALIAS_FLASH_LITE,
  GEMINI_MODEL_ALIAS_PRO,
  GEMMA_4_26B_A4B_IT_MODEL,
  GEMMA_4_31B_IT_MODEL,
  GEMMA_4_TOKEN_LIMIT,
  GEN_AI_AGENT_DESCRIPTION,
  GEN_AI_AGENT_NAME,
  GEN_AI_CONVERSATION_ID,
  GEN_AI_INPUT_MESSAGES,
  GEN_AI_OPERATION_NAME,
  GEN_AI_OUTPUT_MESSAGES,
  GEN_AI_PROMPT_NAME,
  GEN_AI_REQUEST_MODEL,
  GEN_AI_RESPONSE_MODEL,
  GEN_AI_SYSTEM_INSTRUCTIONS,
  GEN_AI_TOOL_CALL_ID,
  GEN_AI_TOOL_DEFINITIONS,
  GEN_AI_TOOL_DESCRIPTION,
  GEN_AI_TOOL_NAME,
  GEN_AI_USAGE_INPUT_TOKENS,
  GEN_AI_USAGE_OUTPUT_TOKENS,
  GET_INTERNAL_DOCS_TOOL_NAME,
  GLOB_DISPLAY_NAME,
  GLOB_TOOL_NAME,
  GOOGLE_ACCOUNTS_FILENAME,
  GOVERNANCE_FILES,
  GREP_DISPLAY_NAME,
  GREP_PARAM_AFTER,
  GREP_PARAM_BEFORE,
  GREP_PARAM_CONTEXT,
  GREP_PARAM_EXCLUDE_PATTERN,
  GREP_PARAM_FIXED_STRINGS,
  GREP_PARAM_INCLUDE_PATTERN,
  GREP_PARAM_MAX_MATCHES_PER_FILE,
  GREP_PARAM_NAMES_ONLY,
  GREP_PARAM_NO_IGNORE,
  GREP_PARAM_TOTAL_MAX_MATCHES,
  GREP_TOOL_NAME,
  GcpLogExporter,
  GcpMetricExporter,
  GcpTraceExporter,
  GeminiChat,
  GeminiCliOperation,
  GeminiClient,
  GeminiEventType,
  GeminiLiveTranscriptionProvider,
  GenAiOperationName,
  GenAiProviderName,
  GenAiTokenType,
  GitIgnoreParser,
  GitService,
  GlobTool,
  GrepTool,
  HOOKS_CONFIG_FIELDS,
  HighWaterMarkTracker,
  HookAggregator,
  HookEventHandler,
  HookEventName,
  HookPlanner,
  HookRegistry,
  HookRunner,
  HookSystem,
  HookType,
  IDEConnectionStatus,
  IDE_DEFINITIONS,
  IDE_MAX_OPEN_FILES,
  IDE_MAX_SELECTED_TEXT_LENGTH,
  IDE_REQUEST_TIMEOUT_MS,
  INCOMPLETE_JSON_MESSAGE,
  INITIAL_HISTORY_LENGTH,
  INTEGRITY_FILENAME,
  INTEGRITY_KEY_FILENAME,
  IdeClient,
  IdeConnectionEvent,
  IdeConnectionType,
  IdeContextNotificationSchema,
  IdeContextSchema,
  IdeContextStore,
  IdeDiffAcceptedNotificationSchema,
  IdeDiffClosedNotificationSchema,
  IdeDiffRejectedNotificationSchema,
  InProcessCheckerType,
  IneligibleTierError,
  IneligibleTierReasonCode,
  InitiationMethod,
  InjectionService,
  IntegrityDataStatus,
  IntegrityStatus,
  IntegrityStoreSchema,
  InvalidNumericProjectIdError,
  InvalidStreamError,
  JsonFormatter,
  JsonStreamEventType,
  KEYCHAIN_SERVICE_NAME,
  KEYCHAIN_TEST_PREFIX,
  KeychainSchema,
  KeychainService,
  KeychainTokenStorage,
  Kind,
  LIST_MCP_RESOURCES_TOOL_NAME,
  LIVE_OUTPUT_MAX_BUFFER_CHARS,
  LSTool,
  LS_DISPLAY_NAME,
  LS_PARAM_IGNORE,
  LS_TOOL_NAME,
  LegacyAgentProtocol,
  LegacyAgentSession,
  LlmRole,
  LoadedTrustedFolders,
  LocalAgentExecutor,
  LocalSandboxManager,
  Logger,
  LoggingContentGenerator,
  MAX_FILE_SIZE_MB,
  MAX_HISTORY_MESSAGES,
  MAX_LINE_LENGTH_TEXT_FILE,
  MAX_TOOL_OUTPUT_SIZE,
  MCPDiscoveryState,
  MCPOAuthProvider,
  MCPOAuthTokenStorage,
  MCPServerConfig,
  MCPServerStatus,
  MCP_DEFAULT_TIMEOUT_MSEC,
  MCP_EXCLUDED_PRIORITY,
  MCP_QUALIFIED_NAME_SEPARATOR,
  MCP_TOOL_PREFIX,
  MEDIA_FILE_PATTERNS,
  MIN_CREDIT_BALANCE,
  MODES_BY_PERMISSIVENESS,
  MUTATOR_KINDS,
  McpClient,
  McpConfigDefinitionSchema,
  MemoryContextManager,
  MemoryMetricType,
  MemoryMonitor,
  MessageBus,
  MessageBusType,
  MessageSenderType,
  ModelConfigService,
  ModelNotFoundError,
  ModelRouterService,
  ModelSlashCommandEvent,
  NO_EDITOR_AVAILABLE_ERROR,
  NetworkRetryAttemptEvent,
  NodeType,
  NoopSandboxManager,
  NotificationType,
  OAuthUtils,
  OUTPUT_UPDATE_INTERVAL_MS,
  OVERAGE_ELIGIBLE_MODELS,
  OnboardUserStatusCode,
  OnboardingStartEvent,
  OnboardingSuccessEvent,
  OpenDiffRequestSchema,
  OutputFormat,
  OverageMenuShownEvent,
  OverageOptionSelectedEvent,
  PARAM_CASE_SENSITIVE,
  PARAM_DESCRIPTION,
  PARAM_DIR_PATH,
  PARAM_FILE_FILTERING_OPTIONS,
  PARAM_FILE_PATH,
  PARAM_PATTERN,
  PARAM_RESPECT_GEMINI_IGNORE,
  PARAM_RESPECT_GIT_IGNORE,
  PLAN_MODE_PARAM_REASON,
  PLAN_MODE_TOOLS,
  PREVIEW_GEMINI_3_1_CUSTOM_TOOLS_MODEL,
  PREVIEW_GEMINI_3_1_FLASH_LITE_MODEL,
  PREVIEW_GEMINI_3_1_MODEL,
  PREVIEW_GEMINI_FLASH_MODEL,
  PREVIEW_GEMINI_MODEL,
  PREVIEW_GEMINI_MODEL_AUTO,
  PRIORITY_SUBAGENT_TOOL,
  PRIORITY_YOLO_ALLOW_ALL,
  PROJECT_MEMORY_INDEX_FILENAME,
  PYTHON_EXCLUDES,
  PerformanceMetricType,
  PlanErrorMessages,
  PolicyDecision,
  PolicyEngine,
  PolicyIntegrityManager,
  PreCompressTrigger,
  PrivateIpError,
  ProjectIdRequiredError,
  QuestionType,
  READ_FILE_DISPLAY_NAME,
  READ_FILE_PARAM_END_LINE,
  READ_FILE_PARAM_START_LINE,
  READ_FILE_TOOL_NAME,
  READ_MANY_FILES_DISPLAY_NAME,
  READ_MANY_FILES_TOOL_NAME,
  READ_MANY_PARAM_EXCLUDE,
  READ_MANY_PARAM_INCLUDE,
  READ_MANY_PARAM_RECURSIVE,
  READ_MANY_PARAM_USE_DEFAULT_EXCLUDES,
  READ_MCP_RESOURCE_TOOL_NAME,
  READ_ONLY_KINDS,
  REDIRECTION_NAMES,
  REFERENCE_CONTENT_END,
  REFERENCE_CONTENT_START,
  RELEASE_CHANNEL_STABILITY,
  ROOT_SCHEDULER_ID,
  RateLimiter,
  ReadFileTool,
  ReadManyFilesTool,
  RecordingContentGenerator,
  ReleaseChannel,
  RequiredMcpServerConfigSchema,
  ResourceRegistry,
  RetryableQuotaError,
  RewindEvent,
  RipGrepTool,
  SCROLLBACK_LIMIT,
  SECRET_FILES,
  SECRET_KEY_ACCOUNT,
  SECRET_PREFIX,
  SERVICE_DESCRIPTION,
  SERVICE_NAME,
  SESSION_FILE_PREFIX,
  SHADOW_REPO_AUTHOR_EMAIL,
  SHADOW_REPO_AUTHOR_NAME,
  SHELL_PARAM_COMMAND,
  SHELL_PARAM_IS_BACKGROUND,
  SHELL_TOOL_NAME,
  SHELL_TOOL_NAMES,
  SKILL_PARAM_NAME,
  SUBAGENT_CANCELLED_ERROR_MESSAGE,
  SUBAGENT_REJECTED_ERROR_PREFIX,
  SYNTHETIC_THOUGHT_SIGNATURE,
  SYSTEM_FILE_EXCLUDES,
  SandboxedFileSystemService,
  Scheduler,
  SchemaValidator,
  SemanticAttributes,
  SessionEndReason,
  SessionStartSource,
  ShellExecutionService,
  ShellTool,
  ShellToolInvocation,
  SimpleExtensionLoader,
  SkillManager,
  SlashCommandStatus,
  SnapshotGenerator,
  SpanStatusCode,
  StandardFileSystemService,
  StartSessionEvent,
  StartupProfiler,
  Storage,
  StreamEventType,
  StreamJsonFormatter,
  SubagentActivityErrorType,
  SubagentState,
  TASK_TYPE_LABELS,
  TODOS_ITEM_PARAM_DESCRIPTION,
  TODOS_ITEM_PARAM_STATUS,
  TODOS_PARAM_TODOS,
  TOOLS_REQUIRING_NARROWING,
  TOOL_LEGACY_ALIASES,
  TOOL_OUTPUTS_DIR,
  TOPIC_PARAM_STRATEGIC_INTENT,
  TOPIC_PARAM_SUMMARY,
  TOPIC_PARAM_TITLE,
  TRACKER_ADD_DEPENDENCY_TOOL_NAME,
  TRACKER_CREATE_TASK_TOOL_NAME,
  TRACKER_GET_TASK_TOOL_NAME,
  TRACKER_LIST_TASKS_TOOL_NAME,
  TRACKER_UPDATE_TASK_TOOL_NAME,
  TRACKER_VISUALIZE_TOOL_NAME,
  TRUSTED_FOLDERS_FILENAME,
  TRUSTED_MCP_SERVER_PRIORITY,
  TaskStatus,
  TaskStatusSchema,
  TaskType,
  TaskTypeSchema,
  TelemetryTarget,
  TerminalQuotaError,
  ToolCallDecision,
  ToolCallEvent,
  ToolConfirmationOutcome,
  ToolErrorType,
  ToolExecutionPhase,
  ToolExecutor,
  ToolOutputTruncatedEvent,
  ToolRegistry,
  TrackerAddDependencyTool,
  TrackerCreateTaskTool,
  TrackerGetTaskTool,
  TrackerListTasksTool,
  TrackerService,
  TrackerTaskSchema,
  TrackerUpdateTaskTool,
  TrackerVisualizeTool,
  TranscriptionFactory,
  TrustLevel,
  Turn,
  UPDATE_TOPIC_DISPLAY_NAME,
  UPDATE_TOPIC_TOOL_NAME,
  UPGRADE_URL_PAGE,
  USER_POLICY_TIER,
  USER_STEERING_INSTRUCTION,
  UiTelemetryService,
  UnauthorizedError,
  UserAccountManager,
  UserPromptEvent,
  UserTierId,
  VALID_GEMINI_MODELS,
  ValidationCancelledError,
  ValidationRequiredError,
  ValueType,
  WEB_FETCH_DISPLAY_NAME,
  WEB_FETCH_PARAM_PROMPT,
  WEB_FETCH_TOOL_NAME,
  WEB_SEARCH_DISPLAY_NAME,
  WEB_SEARCH_PARAM_QUERY,
  WEB_SEARCH_TOOL_NAME,
  WORKSPACE_POLICY_TIER,
  WRITE_FILE_DISPLAY_NAME,
  WRITE_FILE_PARAM_CONTENT,
  WRITE_FILE_TOOL_NAME,
  WRITE_TODOS_TOOL_NAME,
  WarningPriority,
  WebFetchFallbackAttemptEvent,
  WebFetchTool,
  WebSearchTool,
  WhisperModelManager,
  WhisperTranscriptionProvider,
  WindowsSandboxManager,
  WorkspaceContext,
  WorktreeService,
  WriteFileTool,
  WriteTodosTool,
  _clearCache,
  addMCPStatusChangeListener,
  allowEditorTypeInSandbox,
  appendToLastTextPart,
  applyAdminAllowlist,
  applyInboxMemoryPatch,
  applyInboxPatch,
  applyReplacement,
  applyRequiredServers,
  assertValidPathString,
  assumeExhaustive,
  authEvents,
  belongsInConfirmationQueue,
  buildG1Url,
  buildTodosReturnDisplay,
  buildToolResponseData,
  buildToolVisibilityContext,
  buildUserSteeringHintPrompt,
  bytesToMB,
  calculateReplacement,
  captureHeapSnapshot,
  categorizeAndConcatenate,
  checkExhaustive,
  checkPathTrust,
  checkPolicy,
  classifyGoogleError,
  cleanupWorktree,
  clearApiKey,
  clearCachedCredentialFile,
  clearEmittedPolicyWarnings,
  clearOauthClientCache,
  clearRealPathCacheForTesting,
  cloneFunctionCall,
  cloneFunctionResponse,
  clonePart,
  computeModelAddedAndRemovedLines,
  concatenateInstructions,
  connectAndDiscover,
  connectToMcpServer,
  contentPartsToGeminiParts,
  convertColorToHex,
  convertGithubUrlToRaw,
  convertSessionToClientHistory,
  convertToFunctionResponse,
  convertToRestPayload,
  coreEvents,
  createCache,
  createCodeAssistContentGenerator,
  createContentGenerator,
  createContentGeneratorConfig,
  createConversationOffered,
  createHookOutput,
  createPolicyEngineConfig,
  createPolicyUpdater,
  createSafeProxyAgent,
  createSandboxManager,
  createSessionId,
  createTranslationState,
  createTransport,
  createUnauthorizedToolError,
  createWorkingStdio,
  createWorktree,
  createWorktreeService,
  debugLogger,
  decodeTagName,
  deduplicateAbsolutePaths,
  deduplicatePathsByFileIdentity,
  deleteSessionArtifactsAsync,
  deleteSessionFileAndArtifacts,
  deleteStoredSession,
  deleteSubagentSessionDirAndArtifactsAsync,
  deriveSessionShortId,
  detectBOM,
  detectCommandSubstitution,
  detectFileType,
  detectIdeFromEnv,
  detectLineEnding,
  disableBracketedPasteMode,
  disableKittyKeyboardProtocol,
  disableLineWrapping,
  disableModifyOtherKeys,
  disableMouseEvents,
  disableMouseTracking,
  discoverMcpTools,
  discoverPrompts,
  discoverResources,
  discoverTools,
  dismissInboxMemoryPatch,
  dismissInboxPatch,
  dismissInboxSkill,
  displayContentToString,
  doesToolInvocationMatch,
  enableBracketedPasteMode,
  enableKittyKeyboardProtocol,
  enableLineWrapping,
  enableModifyOtherKeys,
  enableMouseEvents,
  encodeTagName,
  ensureStableToolIds,
  enterAlternateScreen,
  escapePath,
  escapeShellArg,
  estimateContextBreakdown,
  execStreaming,
  exitAlternateScreen,
  extractExtensionsFromPatterns,
  extractStringFromParseEntry,
  fetchAdminControls,
  fetchAdminControlsOnce,
  fetchWithTimeout,
  fileExists,
  filter,
  findGitRoot,
  findSecretFiles,
  flatMapTextParts,
  flattenMemory,
  flushTelemetry,
  formatBackgroundCompletionForModel,
  formatBytes,
  formatCheckpointDisplayList,
  formatForSpeech,
  formatMcpToolName,
  formatPolicyError,
  formatProtoJsonDuration,
  formatTruncatedToolOutput,
  formatUserHintsForModel,
  geminiPartsToContentParts,
  generalistProfile,
  generateCheckpointFileName,
  generateFastAckText,
  generateSteeringAckMessage,
  generateSummary,
  generateValidName,
  getAbsoluteGitDir,
  getActivityDetector,
  getActivityMonitor,
  getAdminBlockedMcpServersMessage,
  getAdminErrorMessage,
  getAgentCardLoadOptions,
  getAllGeminiMdFilenames,
  getAllMCPServerStatuses,
  getAllowedMemoryPatchRoots,
  getAlwaysAllowPriorityFraction,
  getApprovalModeDescription,
  getAuthTypeFromEnv,
  getAutoModelDescription,
  getAvailablePort,
  getChannelFromVersion,
  getCheckpointInfoList,
  getCitations,
  getCodeAssistServer,
  getCommandName,
  getCommandRoot,
  getCommandRoots,
  getCompatibilityWarnings,
  getCompressionPrompt,
  getConsentForOauth,
  getConventionAttributes,
  getCoreSystemPrompt,
  getCorrectedFileContent,
  getCurrentGeminiMdFilename,
  getDiffCommand,
  getDirectoryContextString,
  getDisplayString,
  getEditorCommand,
  getEditorDisplayName,
  getEditorExtraArgs,
  getEditorWaitFlag,
  getEnvironmentContext,
  getEnvironmentMemoryPaths,
  getErrorMessage,
  getErrorReplaceResult,
  getErrorStatus,
  getErrorType,
  getEventLoopMonitor,
  getExperiments,
  getExtensionMemoryPaths,
  getFileDiffFromResultDisplay,
  getFolderStructure,
  getFsErrorMessage,
  getFunctionCalls,
  getFunctionCallsAsJson,
  getFunctionCallsFromParts,
  getFunctionCallsFromPartsAsJson,
  getG1CreditBalance,
  getGlobalMemoryFilePath,
  getGlobalMemoryPaths,
  getHookKey,
  getHookSource,
  getIdeInstaller,
  getInitialChatHistory,
  getMCPDiscoveryState,
  getMCPServerPrompts,
  getMCPServerStatus,
  getMatchingSessionFiles,
  getMemoryMonitor,
  getOauthClient,
  getPackageJson,
  getPlanModeExitMessage,
  getPolicyDenialError,
  getPolicyDirectories,
  getPolicyTier,
  getPreviousSession,
  getProjectHash,
  getProjectMemoryIndexFilePath,
  getProjectRootForWorktree,
  getPromptIdWithFallback,
  getPty,
  getRealPath,
  getReleaseChannel,
  getRemoteAgentTargetUrl,
  getResponseText,
  getResponseTextFromParts,
  getRetryErrorType,
  getSecretFileFindArgs,
  getShellConfiguration,
  getSpecificMimeType,
  getStructuredResponse,
  getStructuredResponseFromParts,
  getToolAliases,
  getToolCallDataSchema,
  getToolSuggestion,
  getTruncatedCheckpointNames,
  getUserProjectMemoryPaths,
  getVersion,
  getWorktreePath,
  handleFallback,
  hasCycleInSchema,
  hasNetworkTransport,
  hasRedirection,
  hasSummary,
  hasValidEditorCommand,
  hasValidEditorCommandAsync,
  hasWorktreeChanges,
  homedir,
  ideContextStore,
  initializeActivityMonitor,
  initializeMemoryMonitor,
  initializeShellParsers,
  initializeTelemetry,
  invokeMcpPrompt,
  isAbortError,
  isAccountSuspendedError,
  isActiveModel,
  isAddressPrivate,
  isAgentThought,
  isAgentYield,
  isApiError,
  isAppleTerminal,
  isAuthenticationError,
  isAutoModel,
  isBackgroundExecutionData,
  isBinary,
  isBinaryAvailable,
  isBinaryFile,
  isCloudShell,
  isCompletedAskUserTool,
  isCustomModel,
  isDumbTerminal,
  isEditToolParams,
  isEditorAvailable,
  isEditorAvailableAsync,
  isEmpty,
  isEnabled,
  isFatalToolError,
  isFileDiff,
  isGemini2Model,
  isGemini3Model,
  isGeminiWorktree,
  isGitRepository,
  isGnuScreen,
  isGrepResult,
  isGuiEditor,
  isHeadlessMode,
  isInvalidArgumentError,
  isJetBrainsTerminal,
  isListResult,
  isLoopbackHost,
  isLowColorTmux,
  isMaskedTool,
  isMcpToolAnnotation,
  isMcpToolName,
  isNightly,
  isNodeError,
  isOverageEligibleModel,
  isPerformanceMonitoringActive,
  isPreview,
  isPreviewModel,
  isPrivateIp,
  isPrivateIpAsync,
  isProModel,
  isProjectSkillPatchTarget,
  isReadManyFilesResult,
  isRecord,
  isRenderedInHistory,
  isRetryableError,
  isRollingSummary,
  isSchemaDepthError,
  isSecretFile,
  isSnapshot,
  isStable,
  isStructuredError,
  isStructuredToolResult,
  isSubagentProgress,
  isSubpath,
  isSystemEvent,
  isTelemetrySdkInitialized,
  isTerminalEditor,
  isTextPart,
  isTmux,
  isTool,
  isToolActivityError,
  isToolCallResponseInfo,
  isToolExecution,
  isTrustLevel,
  isTrustedSystemPath,
  isUserActive,
  isUserPrompt,
  isUserVisibleHook,
  isValidEditorType,
  isValidNonThoughtTextPart,
  isValidPathString,
  isValidToolName,
  isVisibleInToolGroup,
  isWindows,
  isWindows10,
  isWithinRoot,
  isWriteFileToolParams,
  jsonToMarkdown,
  generalistProfile2 as legacyGeneralistProfile,
  listExtensions,
  listInboxMemoryPatches,
  listInboxPatches,
  listInboxSkills,
  listMemoryFiles,
  loadAgentsFromDirectory,
  loadApiKey,
  loadConversationRecord,
  loadExtensionPolicies,
  loadJitSubdirectoryMemory,
  loadPoliciesFromToml,
  loadSkillFromFile,
  loadSkillsFromDir,
  loadTrustedFolders,
  loadWasmBinary,
  logApiError,
  logApiRequest,
  logApiResponse,
  logBillingEvent,
  logChatCompression,
  logCliConfiguration,
  logConsecaPolicyGeneration,
  logConsecaVerdict,
  logConversationFinishedEvent,
  logExtensionDisable,
  logExtensionEnable,
  logExtensionInstallEvent,
  logExtensionUninstall,
  logExtensionUpdateEvent,
  logFlashFallback,
  logIdeConnection,
  logModelSlashCommand,
  logNetworkRetryAttempt,
  logOnboardingStart,
  logOnboardingSuccess,
  logRewind,
  logSlashCommand,
  logToolCall,
  logToolOutputTruncated,
  logUserPrompt,
  logWebFetchFallbackAttempt,
  makeChatCompressionEvent,
  makeFakeConfig,
  makeRelative,
  makeSlashCommandEvent,
  mapError,
  mapFinishReason,
  mapHttpToGrpcStatus,
  mapUsage,
  markdownToAgentDefinition,
  mcpServerRequiresOAuth,
  moveInboxSkill,
  normalizeCommand,
  normalizePath,
  normalizeSpace,
  normalizeUrl,
  openBrowserSecurely,
  openDiff,
  parseAgentMarkdown,
  parseAndFormatApiError,
  parseBashCommandDetails,
  parseBooleanEnvFlag,
  parseCommandDetails,
  parseFrontmatter,
  parseGoogleApiError,
  parseMcpToolName,
  parsePrompt,
  parseTelemetryTargetValue,
  parseThought,
  partListUnionToString,
  partToString,
  patchStdio,
  performInit,
  performRestore,
  populateMcpServerCommand,
  populateToolDisplay,
  processRestorableToolCalls,
  processSingleFileContent,
  promptIdContext,
  readFileWithEncoding,
  readGeminiMdFiles,
  readPathFromWorkspace,
  readPolicyFiles,
  readWasmBinaryFromDisk,
  recordApiErrorMetrics,
  recordApiRequestBreakdown,
  recordApiResponseMetrics,
  recordBaselineComparison,
  recordContentRetry,
  recordContentRetryFailure,
  recordConversationOffered,
  recordCpuUsage,
  recordCreditPurchaseClick,
  recordCurrentMemoryUsage,
  recordCustomApiResponseMetrics,
  recordCustomTokenUsageMetrics,
  recordEventLoopDelay,
  recordExitFail,
  recordFileOperationMetric,
  recordFlickerFrame,
  recordGenAiClientOperationDuration,
  recordGenAiClientTokenUsage,
  recordInvalidChunk,
  recordMemoryUsage,
  recordModelRoutingMetrics,
  recordOverageOptionSelected,
  recordPerformanceRegression,
  recordPerformanceScore,
  recordRetryAttemptMetrics,
  recordSlowRender,
  recordStartupPerformance,
  recordTokenEfficiency,
  recordTokenUsageMetrics,
  recordToolCallInteractions,
  recordToolCallMetrics,
  recordToolExecutionBreakdown,
  recordToolQueueDepth,
  recordUserActivity,
  refreshMemory,
  removeMCPStatusChangeListener,
  renderDisplayDiff,
  resetApiKeyCacheForTesting,
  resetBrowserSession,
  resetGeminiMdFilename,
  resetOauthClientForTesting,
  resetTrustedFoldersForTesting,
  resetUserDataCacheForTesting,
  resetVersionCache,
  resolveAndValidatePlanPath,
  resolveAtCommandPath,
  resolveClassifierModel,
  resolveEditorAsync,
  resolveEditorTypeFromCommand,
  resolveExecutable,
  resolveModel,
  resolvePathFromEnv,
  resolveRipgrepPath,
  resolveSandboxPaths,
  resolveTelemetrySettings,
  resolveToRealPath,
  retryWithBackoff,
  runInDevTraceSpan,
  safeJsonToMarkdown,
  safeLiteralReplace,
  safeTemplateReplace,
  sanitizeAdminSettings,
  sanitizeFilenamePart,
  sanitizeHostname,
  sanitizeOutput,
  saveApiKey,
  saveTruncatedToolOutput,
  saveTrustedFolders,
  scheduleAgentTools,
  serializeTerminalToObject,
  setGeminiMdFilename,
  setGlobalProxy,
  setupUser,
  shortenPath,
  shouldAttemptBrowserLaunch,
  shouldAutoUseCredits,
  shouldEnterAlternateScreen,
  shouldLaunchBrowser,
  shouldShowEmptyWalletMenu,
  shouldShowOverageMenu,
  showMemory,
  shutdownTelemetry,
  sortFileEntries,
  spawnAsync,
  splitCommands,
  startGlobalActivityMonitoring,
  startGlobalEventLoopMonitoring,
  startGlobalMemoryMonitoring,
  startMemoryService,
  startupProfiler,
  stopAdminControlsPolling,
  stopGlobalActivityMonitoring,
  stopGlobalEventLoopMonitoring,
  stopGlobalMemoryMonitoring,
  stressTestProfile,
  stripAnsiFromBuffer,
  stripHupGuard,
  stripShellWrapper,
  stripToolCallIdPrefixes,
  supports256Colors,
  supportsModernFeatures,
  supportsMultimodalFunctionResponse,
  supportsTrueColor,
  tildeifyPath,
  tmpdir,
  toAbsolutePath,
  toFriendlyError,
  toPathKey,
  tokenLimit,
  toolResultDisplayToDisplayContent,
  translateEvent,
  truncateFastAckInput,
  truncateString,
  uiTelemetryService,
  unescapePath,
  updateGlobalFetchTimeouts,
  updateMCPServerStatus,
  updatePart,
  updatePolicy,
  validateAndSanitizeSessionId,
  validateMcpPolicyToolNames,
  validatePatches,
  validatePath,
  validatePlanContent,
  validatePlanPath,
  wrapInAccountChooser,
  writeToStderr,
  writeToStdout
};
