/**
 * @license
 * Copyright 2025 Google LLC
 * SPDX-License-Identifier: Apache-2.0
 */
import { type DOMElement } from './dom.js';
export declare function getScrollHeight(node: DOMElement): number;
export declare function getScrollWidth(node: DOMElement): number;
export declare function calculateScroll(node: DOMElement, isTerminalResized?: boolean): void;
export declare function getScrollTop(node: DOMElement): number;
export declare function getScrollLeft(node: DOMElement): number;
