/**
Wrapper for non-Error values that were thrown.

In JavaScript, any value can be thrown (not just Error instances). This class wraps such values to ensure consistent error handling.
*/
export declare class NonError extends Error {
    name: string;
    readonly value: unknown;
    constructor(value: unknown);
}
