import { StyledLine } from './styled-line.js';
export declare class Serializer {
    private buffer;
    private currentSize;
    constructor(initialSize?: number);
    serialize(lines: readonly StyledLine[]): Uint8Array;
    private ensureCapacity;
    private writeLine;
    private writeSpan;
    private writeString;
    private writeUint32;
    private writeUint8;
}
export declare class Deserializer {
    private offset;
    private readonly buf;
    constructor(buffer: Uint8Array);
    deserialize(): StyledLine[];
    private readLine;
    private readSpan;
    private readString;
    private readUint32;
    private readUint16;
    private readUint8;
}
