import { type ScrollbarBoundingBox } from './measure-element.js';
export type ScrollbarThumb = {
    startIndex: number;
    endIndex: number;
    thumbStartHalf: number;
    thumbEndHalf: number;
};
export type DrawOptions = {
    layout: ScrollbarBoundingBox;
    clip: {
        x: number;
        y: number;
        w: number;
        h: number;
    };
    axis: 'vertical' | 'horizontal';
    color?: string;
    setChar: (x: number, y: number, value: string, formatFlags: number, fgColor?: string, bgColor?: string, link?: string) => void;
    getExistingChar?: (x: number, y: number) => {
        bgColor?: string;
    } | undefined;
};
export declare const renderScrollbar: ({ layout, clip, axis, color, setChar, getExistingChar, }: DrawOptions) => void;
