import {IArrayLike} from '../utils/types';

export function inplaceQuickSort(array: IArrayLike, lo: number, hi: number): IArrayLike;
export function inplaceQuickSortIndices(array: IArrayLike, indices: IArrayLike, lo: number, hi: number): IArrayLike;
